// Code for Useless Machine: CDDF, SS 2024 
// Chiu YuLun, Kotrozou Athina, Eris Rakipaj
// Confrontation Avoidant Machine  / a machine that you try to speak to, but reacts to the tone of youe voice 
// -- an honest attempt < 3

#include <stdint.h>
#include <math.h>

// Constant variables: 
const int leftFwd = 2;
const int leftBwd = 3;
const int rightFwd = 4;
const int rightBwd = 5;

// Mics:
const int rightMic = A0;
const int leftMic = A1;

// Other variables
const int sample_window = 400; // amount of time to sample sound in milliseconds
int MicR;      // microphone 1 value from analogRead (0-1023)
int MicL;      // microphone 2 value from analogRead (0-1023)
int rightMic_max;  // maximum value recorded by mic1 during sample window
int rightMic_min;  // minimum value recorded by mic1 during sample window
int leftMic_max;  // maximum value recorded by mic2 during sample window
int leftMic_min;  // minimum value recorded by mic2 during sample window
int rightAmp;      // largest amplitude of mic1 during sample window (max-min)
int leftAmp;      // largest amplitude of mic2 during sample window (max-min)
unsigned long start_time;  // time in milliseconds at start of sample window (since program started)
unsigned long current_time; // current time in milliseconds (since program started)
//const int facingThreshold = 10; // threshold for detecting if facing the sound source

struct ListenResults {
  int difference;
  int avgAmp;
};

//Threshold values
const int SHOUTING_THRESHOLD = 60; // amp loud
const int NORMAL_THRESHOLD = 10; // normal

enum states {
  NONE,
  LISTEN,
  NAVIGATE,
  MOVE
};

states prior_state, state;

void setup() 
{
  pinMode(leftFwd , OUTPUT);
  pinMode(leftBwd , OUTPUT);
  pinMode(rightFwd , OUTPUT);
  pinMode(rightBwd , OUTPUT);

  prior_state = NONE;
  state = LISTEN;

  Serial.begin(9600);
}

void loop() {
 ListenResults results = Listen();
 int difference = results.difference;
 int avgAmp = results.avgAmp;
 StopDriving();
 delay(500);
  switch (state) {
    case LISTEN:
      listen();
      Serial.print(" in listen state. ");
      break;
    case NAVIGATE:
      navigate();
      Serial.print(" in navigate state. ");
      break;
    case MOVE:
      move();
      Serial.print(" in move state. ");
      break;
    }
  }

////State Functions
void listen(){
  // enter listening state
  if (state != prior_state) {
    prior_state = state;
  }
  // perform state actions:
  ListenResults results = Listen(); // Obtain results from Listen() function
  int difference = results.difference; // Assign difference from results
  int avgAmp = results.avgAmp;
  Serial.print("listening");

  // check for transitions to next state
  if (difference != 0) {
    state = NAVIGATE;
    Serial.print("navigating");
  }
}

void navigate(){ 
  // enter navigation state
  if (state != prior_state) {
    prior_state = state;
  }
  ListenResults results = Listen();
  int difference = results.difference;
  int avgAmp = results.avgAmp;
  
  Navigate(difference);

  if ( abs(difference) == 0 && avgAmp >= NORMAL_THRESHOLD ) {
    state = MOVE;
    Serial.print("moving");
  }
  else {
    state = LISTEN;
    Serial.print("listening");
  }
}

void move() {
  //enter move state 
  if (state != prior_state) {
    prior_state = state;
  }

  ListenResults results = Listen();
  int difference = results.difference;
  int avgAmp = results.avgAmp;
  Move(avgAmp);

  
  if (difference != 0) {
    state = LISTEN;
    Serial.print("moving back to listening state");
  }
}

//////////////////////////////////////////////
// Listening function - sound recording:

ListenResults Listen() {
  // Reset max and min values
 
  rightMic_min = 1023;
  rightMic_max = 0;
  leftMic_min = 1023;
  leftMic_max = 0;

  start_time = millis(); // Start time for this sample window
  Serial.println("Beginning sample ");

  while (millis() - start_time < sample_window) {
    // Read mic values
    MicR = analogRead(rightMic);
    MicL = analogRead(leftMic);
 
    // Assign new min and max values
    rightMic_min = min(MicR, rightMic_min);
    rightMic_max = max(MicR, rightMic_max);
    leftMic_min = min(MicL, leftMic_min);
    leftMic_max = max(MicL, leftMic_max);

    current_time = millis(); // Update current time
  }

  // Done sampling - calculate max amplitude for each mic
  rightAmp = rightMic_max - rightMic_min;
  leftAmp = leftMic_max - leftMic_min;
  int avgAmp = (rightAmp + leftAmp) / 2;
  int difference = rightAmp - leftAmp;

  // Print amplitudes to serial monitor
  Serial.print(" Right Mic amplitude: ");
  Serial.print(rightAmp);
  Serial.print(" | Left Mic amplitude: ");
  Serial.print(leftAmp);
  Serial.print(" | Average Amplitude = ");
  Serial.println(avgAmp);
  Serial.print(" | Amplitude Difference = ");
  Serial.println(difference);
 
  ListenResults results = {difference, avgAmp};
  return results;
}

// Movement functions in if statements
void Navigate(int difference) {
  if (difference > 0) {
    TurnRight();
    //delay(difference*100);
    delay(100);
    StopDriving();
    delay(50);
  }
  else if (difference < 0) {
    TurnLeft();
    //delay(difference*100);
    delay(100);
    StopDriving();
    delay(50);
  }
  else {
    StopDriving();
    delay(500);
  }
  
}

void Move(int avgAmp) {
  if (avgAmp > SHOUTING_THRESHOLD ) {
    TurnRight();
    delay(500);
    StopDriving();
    delay(500);
    MoveForward();
    delay(1000);
    StopDriving();
  }

  else {
    MoveForward();
    delay(100);
    StopDriving();
  }
}



// Movement functions:
void StopDriving() {  // Set pins to make robot stop
  digitalWrite(leftFwd, LOW);
  digitalWrite(leftBwd, LOW);
  digitalWrite(rightFwd, LOW);
  digitalWrite(rightBwd, LOW);
  Serial.print("stopped");
}

void MoveForward() { // Set pins to move forward
  digitalWrite(leftFwd, HIGH);
  digitalWrite(leftBwd, LOW);
  digitalWrite(rightFwd, HIGH);
  digitalWrite(rightBwd, LOW);
  Serial.print("moving front");
}

void MoveBackward() { // Set pins to move backward
  digitalWrite(leftFwd, LOW);
  digitalWrite(leftBwd, HIGH);
  digitalWrite(rightFwd, LOW);
  digitalWrite(rightBwd, HIGH);
  Serial.print("moving back");
}

void TurnLeft() { // Set pins to turn left
  digitalWrite(leftFwd, HIGH);
  digitalWrite(rightBwd, HIGH);
  digitalWrite(leftBwd, LOW);
  digitalWrite(rightFwd, LOW);
  Serial.print("turning left");
}

void TurnRight() { // Set pins to turn right
  digitalWrite(leftBwd, HIGH);
  digitalWrite(rightFwd, HIGH);
  digitalWrite(leftFwd, LOW);
  digitalWrite(rightBwd, LOW);
  Serial.print("turning right");
}
